import re
import datetime

def parse_interval(val):
    match = re.match(r'^(\d+)([smhdwy])$', val)
    if not match:
        raise ValueError()

    unit = {
        's': 'seconds',
        'm': 'minutes',
        'h': 'hours',
        'd': 'days',
        'w': 'weeks',
    }[match.group(2)]

    td = datetime.timedelta(**{unit: int(match.group(1))})

    return (td.days * 86400) + td.seconds



# -*- coding: utf-8 -*-
#
# django-dumpslow -- Django application to log and summarize slow requests
#                    <http://chris-lamb.co.uk/projects/django-dumpslow>
#
# Copyright © 2009-2011 Chris Lamb <chris@chris-lamb.co.uk>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

import re
import time
import redis

from operator import itemgetter
from optparse import make_option

from django.conf import settings
from django.core.management.base import NoArgsCommand, CommandError

from django_dumpslow.utils import parse_interval

class Command(NoArgsCommand):
    help = "Parse and summarize the django-dumpslow slow request log"

    option_list = NoArgsCommand.option_list + (
        make_option('-s', dest='order', metavar='ORDER', default='at',
            help="what to sort by (at, count, average) (default: at)"),
        make_option('-i', dest='after', metavar='INTERVAL', default=0,
            help="interval to report on (eg. 3d 1w 1y) (default: all)"),
        make_option('-r', dest='reverse', default=False, action='store_true',
            help="reverse the sort order (largest last instead of first)"),
        make_option('-t', dest='limit', default=None, metavar='NUM',
            help="just show the top NUM queries"),
        make_option('-m', dest='max_duration', metavar='SECS', default=20,
            help="ignore entries over SECS seconds (default: 20)"),
    )

    def handle(self, **options):
        def check_option(name, val):
            try:
                val = int(val)
                if val < 0:
                    raise ValueError()
                return val
            except ValueError:
                raise CommandError(
                    'Invalid value for %s %r' % (name, val)
                )
            except TypeError:
                pass

        limit = check_option('-t', options['limit'])
        max_duration = check_option('-m', options['max_duration'])

        after = options['after']
        if after:
            try:
                after = int(time.time()) - parse_interval(after)
            except ValueError:
                raise CommandError('Invalid interval %r' % after)

        order = options['order']
        if order not in ('at', 'count', 'average'):
            raise CommandError('Invalid sort order %r' % options['order'])

        client = redis.Redis(
            host=settings.REDIS_HOST,
            port=settings.REDIS_PORT,
        )

        data = {}
        totals = {}
        hits = {}
        results = client.zrangebyscore(
            getattr(settings, 'DUMPSLOW_REDIS_KEY', 'dumpslow'), after, '+inf',
        )

        for line in results:
            view, duration = line.split('\n', 1)

            duration = float(duration)

            if max_duration and duration >= max_duration:
                continue

            if order == 'at':
                try:
                    data[view] += duration
                except KeyError:
                    data[view] = duration

            elif order == 'count':
                try:
                    data[view] += 1
                except KeyError:
                    data[view] = 1

            elif order == 'average':
                try:
                    totals[view] += duration
                    hits[view] += 1
                except KeyError:
                    totals[view] = duration
                    hits[view] = 1

                data[view] = totals[view] / hits[view]


        items = data.items()
        del data
        items.sort(key=itemgetter(1), reverse=not options['reverse'])

        if limit is not None:
            items = items[:limit]

        print "", "View",
        print {
            'count': 'Count',
            'at': 'Accumulated time',
            'average': 'Average time',
        }[order].rjust(66)
        print "", "=" * 71

        for view, value in items:
            pad = 70 - len(view)

            print "", view,
            if order == 'at':
                print ("%2.2f" % value).rjust(pad)
            elif order == 'count':
                print str(value).rjust(pad)
            elif order == 'average':
                print ("%2.2f" % value).rjust(pad)



# -*- coding: utf-8 -*-
#
# django-dumpslow -- Django application to log and summarize slow requests
#                    <http://chris-lamb.co.uk/projects/django-dumpslow>
#
# Copyright © 2009-2011 Chris Lamb <chris@chris-lamb.co.uk>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

import sys
import time
import redis
import threading

from django.conf import settings
from django.core.mail import mail_admins

from django_dumpslow.utils import parse_interval

class LogLongRequestMiddleware(object):
    def __init__(self):
        self.local = threading.local()

    def process_view(self, request, callback, callback_args, callback_kwargs):
        view = '%s.' % callback.__module__

        try:
            view += callback.__name__
        except (AttributeError, TypeError):
            # Some view functions (eg. class-based views) do not have a
            # __name__ attribute; try and get the name of its class
            view += callback.__class__.__name__

        self.local.view = view
        self.local.start_time = time.time()

    def process_response(self, request, response):
        try:
            view = self.local.view
            time_taken = time.time() - self.local.start_time
        except AttributeError:
            # If, for whatever reason, the variables are not available, don't
            # do anything else.
            return response

        if time_taken < getattr(settings, 'DUMPSLOW_LONG_REQUEST_TIME', 1):
            return response

        client = redis.Redis(
            host=settings.REDIS_HOST,
            port=settings.REDIS_PORT,
        )

        client.zadd(
            getattr(settings, 'DUMPSLOW_REDIS_KEY', 'dumpslow'),
            '%s\n%.3f' % (view, time_taken),
            self.local.start_time,
        )

        # Clean up old values

        delete_after = parse_interval(
            getattr(settings, 'DUMPSLOW_DELETE_AFTER', '4w'),
        )

        client.zremrangebyscore(
            getattr(settings, 'DUMPSLOW_REDIS_KEY', 'dumpslow'),
            0,
            int(time.time()) - delete_after,
        )

        # If it was really slow, email admins. Disabled by default.
        email_threshold = getattr(settings, 'DUMPSLOW_EMAIL_REQUEST_TIME', sys.maxint)
        if time_taken > email_threshold:
            mail_admins(
                "SLOW PAGE: %s" % request.path,
                "This page took %2.2f seconds to render, which is over the threshold "
                "threshold of %s.\n\n%s" % (
                    time_taken,
                    email_threshold,
                    str(request),
                ),
            )

        return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# django-dumpslow -- Django application to log and summarize slow requests
#                    <http://chris-lamb.co.uk/projects/django-dumpslow>
#
# Copyright © 2009-2011 Chris Lamb <chris@chris-lamb.co.uk>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

setup_args = dict(
    name='django-dumpslow',
    version='2.0.1',
    license='BSD',
    url='https://chris-lamb.co.uk/projects/django-dumpslow',
    packages=(
        'django_dumpslow',
        'django_dumpslow.management',
        'django_dumpslow.management.commands',
    ),
    author='Chris Lamb',
    author_email='chris@chris-lamb.co.uk',
    description="Django application to log and summarize slow requests",
)

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(**setup_args)
